var My_JsBridge = function () {
    if (window.fund) {
        return;
    }
    var bridgeSyncIdentifier = 'JsBridge_Key_SyncIdentifier';
    var bridgeAsyncIdentifier = 'JsBridge_Key_AsyncIdentifier';

    var callSuccessKey = 'JsBridge_Key_CallSuccess';
    var callFailKey = 'JsBridge_Key_CallFail';
    var callCompleteKey = 'JsBridge_Key_CallComplete';

    var callJsFuncArgKey = 'JsBridge_Key_CallJsFuncArg';

    var log = function (...args) {
        if(FundWebBridge.isDebugEnable()) {
            console.log(...args);
        }
    };

    var jsType = function () {
        var type = {};
        var typeArr = ['String', 'Object', 'Number', 'Array', 'Undefined', 'Function', 'AsyncFunction', 'Null', 'Symbol', 'Boolean'];
        for (var i = 0; i < typeArr.length; i++) {
            (function (name) {
                type['is' + name] = function (obj) {
                    return Object.prototype.toString.call(obj) == '[object ' + name + ']'
                }
            })(typeArr[i])
        }
        return type
    }();
    var callMap_keyId = 0;
    var callMap_data = {};
    var callMap_add = function (randomKey, funcNameKey, func) {
        var funcMap = callMap_data[randomKey];
        if (!jsType.isObject(funcMap)) {
            var map = {};
            map[funcNameKey] = func;
            callMap_data[randomKey] = map
        } else {
            funcMap[funcNameKey] = func;
            callMap_data[randomKey] = funcMap
        }
        return randomKey + funcNameKey
    };
    var callMap_remove = function (randomKey) {
        if (!callMap_data.hasOwnProperty(randomKey)) return;
        delete callMap_data[randomKey]
    };
    var makeParams = function (apiPrefix, moduleName, methodName, params, index) {
        if (!jsType.isObject(params) && !(jsType.isFunction(params) || jsType.isAsyncFunction(params))) {
            return params
        }
        var randomKey = '-' + apiPrefix + '-' + (jsType.isUndefined(moduleName) ? 'Undefined' : moduleName) + '-' + methodName + '-arg' + index + '-' + (new Date).getTime().toString() + '-' + (callMap_keyId++).toString() + '-';
        var newParams = {};
        var funcId = '';
        if (jsType.isFunction(params) || jsType.isAsyncFunction(params)) {
            funcId = callMap_add(randomKey, callJsFuncArgKey, params);
            newParams[callJsFuncArgKey] = funcId;
            return newParams
        }
        newParams = params;
        var success = params.success;
        if (success && (jsType.isFunction(success) || jsType.isAsyncFunction(success))) {
            funcId = callMap_add(randomKey, callSuccessKey, success);
            newParams[callSuccessKey] = funcId
        }
        var fail = params.fail;
        if (fail && (jsType.isFunction(fail) || jsType.isAsyncFunction(fail))) {
            funcId = callMap_add(randomKey, callFailKey, fail);
            newParams[callFailKey] = funcId
        }
        var complete = params.complete;
        if (complete && (jsType.isFunction(complete) || jsType.isAsyncFunction(complete))) {
            funcId = callMap_add(randomKey, callCompleteKey, complete);
            newParams[callCompleteKey] = funcId
        }
        return newParams;
    };
    var sendParams = function (apiPrefix, moduleName, methodName, bridgeIdentifier, params) {
        var newParams = params;
        var resArgs = [];
        if (Object.prototype.toString.call(newParams) === '[object Arguments]') {
            var argCount = newParams.length;
            for (var argIdx = 0; argIdx < argCount; argIdx++) {
                resArgs.push(makeParams(apiPrefix, moduleName, methodName, newParams[argIdx], argIdx))
            }
        }
        return {
            apiPrefix: apiPrefix,
            moduleName: moduleName,
            methodName: methodName,
            bridgeIdentifier: bridgeIdentifier,
            args: resArgs
        };
    };

    var sendNativeAsync = function (params) {
        log("before async callNativeApi: ", JSON.stringify(params));
        FundWebBridge.callNativeApi(JSON.stringify(params));
    };
    var sendNativeSync = function (params) {
        log("before sync callNativeApi: ", JSON.stringify(params));
        var res = FundWebBridge.callNativeApi(JSON.stringify(params));;
        if (jsType.isNull(res)) {
            return undefined
        }
        if (jsType.isString(res) && res.length == 0) {
            return null
        }
        if (!res) return null;
        try {
            log("receive sync callback : ", res);
            res = JSON.parse(res);
            return res.data;
        } catch (error) {
            if ((jsType.isFunction(window.onerror) || jsType.isAsyncFunction(window.onerror)) && Object.prototype.toString.call(error) == '[object Error]') {
                window.onerror.apply(window, [error])
            }
        }
        return null
    };


    function _postMessageToWeb(jsonData) {
        var messageEvent = document.createEvent("Event");
        messageEvent.initEvent("message", true, true);
        var jsonObject = JSON.parse(decodeURIComponent(jsonData));
        messageEvent.data = jsonObject.data;
        messageEvent.origin = jsonObject.origin;
        window.dispatchEvent(messageEvent);
    }


    var receiveNativeCall = function (params) {
        var funcRes = null;
        if (!jsType.isString(params) || !params) {
            return funcRes
        }
        log("receiveNativeCall: ", decodeURIComponent(params));
        var newParams = JSON.parse(decodeURIComponent(params));
        if (!jsType.isObject(newParams)) {
            return funcRes
        }
        var funcId = newParams.funcId;
        var resDatas = jsType.isArray(newParams.data) ? newParams.data : [];
        var alive = newParams.alive;
        var randomKey = '',
            funcNameKey = '';
        var matchKey = function (key) {
            var matchNumber = funcId.length - key.length;
            if (matchNumber >= 0 && funcId.lastIndexOf(key) == matchNumber) { } else {
                return false
            }
            randomKey = funcId.replace(new RegExp(key, 'g'), '');
            funcNameKey = key;
            return true
        };
        var matchRes = matchKey(callSuccessKey) || matchKey(callFailKey) || matchKey(callCompleteKey) || matchKey(callJsFuncArgKey);
        if (!matchRes) return funcRes;
        var funcMap = callMap_data[randomKey];
        if (!jsType.isObject(funcMap)) return funcRes;
        var func = funcMap[funcNameKey];
        if (!(jsType.isFunction(func) || jsType.isAsyncFunction(func))) return funcRes;
        try {
            funcRes = func(...resDatas)
        } catch (error) {
            if ((jsType.isFunction(window.onerror) || jsType.isAsyncFunction(window.onerror)) && Object.prototype.toString.call(error) == '[object Error]') {
                window.onerror.apply(window, [error])
            }
            funcRes = null
        }
        if (alive) return funcRes;
        if (funcNameKey == callCompleteKey || funcNameKey == callJsFuncArgKey) {
            callMap_remove(randomKey)
        }
        return funcRes
    };
    var makeApi = function (apiPrefix, moduleName, apiMap) {
        if (!apiPrefix || !jsType.isString(apiPrefix) || !jsType.isObject(apiMap)) {
            return {}
        }
        var res = {};
        res._postMessageToWeb = _postMessageToWeb;
        var mapKeys = Object.keys(apiMap);
        for (var i = 0; i < mapKeys.length; i++) {
            (function (methodName) {
                var config = apiMap[methodName];
                var isSync = config.hasOwnProperty('sync') ? config.sync : false;
                res[methodName] = isSync ? function () {
                    return sendNativeSync(sendParams(apiPrefix, moduleName, methodName, bridgeSyncIdentifier, arguments))
                } : function () {
                    sendNativeAsync(sendParams(apiPrefix, moduleName, methodName, bridgeAsyncIdentifier, arguments))
                }
            })(mapKeys[i])
        }
        return res
    };
    var makeModuleApi = function (apiPrefix, desApi, moduleApiMap) {
        if (!apiPrefix || !jsType.isString(apiPrefix) || !desApi || !jsType.isObject(desApi) || !moduleApiMap || !jsType.isObject(moduleApiMap)) {
            return desApi
        }
        var resModuleMap = {};
        var mapKeys = Object.keys(moduleApiMap);
        for (var i = 0; i < mapKeys.length; i++) {
            (function (moduleName) {
                resModuleMap[moduleName] = makeApi(apiPrefix, moduleName, moduleApiMap[moduleName])
            })(mapKeys[i])
        }
        var modulesKey = 'registerModules';
        var requireModuleKey = 'requireModule';
        desApi[modulesKey] = function () {
            return resModuleMap
        };
        desApi[requireModuleKey] = function (moduleName) {
            if (!moduleName || !jsType.isString(moduleName)) {
                return undefined
            }
            return resModuleMap[moduleName]
        };
        mapKeys = Object.keys(resModuleMap);
        for (var j = 0; j < mapKeys.length; j++) {
            (function (moduleName) {
                desApi[moduleName] = function () {
                    var moduleFunc = desApi[requireModuleKey];
                    if (!moduleFunc) {
                        return moduleFunc
                    }
                    if (!(jsType.isFunction(moduleFunc) || jsType.isAsyncFunction(moduleFunc))) {
                        return undefined
                    }
                    return moduleFunc(moduleName)
                }
            })(mapKeys[j])
        }
        return desApi
    };
    return [receiveNativeCall, makeApi, makeModuleApi, callMap_data]
}();
var My_JsBridge_ReceiveNativeCall = My_JsBridge[0];
var My_JsBridge_MakeApi = My_JsBridge[1];
var My_JsBridge_MakeModuleApi = My_JsBridge[2];
var callMap_data = My_JsBridge[3];